#include <EEPROM.h>
#include <TM1637Display.h>

// Pins for the 7-segment display
#define CLK_PIN 2
#define DIO_PIN 3

// Pins for buttons
#define HOURS_BUTTON 4
#define MINUTES_BUTTON 5
#define SECONDS_BUTTON 6
#define START_BUTTON 7
#define RESET_BUTTON 8

// Pins for RGB LEDs
#define RED_LED 9
#define GREEN_LED 10
#define BLUE_LED 11

// Relay control pin
#define RELAY_PIN 12

// Time variables
unsigned long previousMillis = 0;
unsigned long interval = 1000; // One second interval
byte hours = 0;
byte minutes = 0;
byte seconds = 0;
byte fractions = 0;

// Timer control variables
bool timerActive = false;

// Display object
TM1637Display display(CLK_PIN, DIO_PIN);

void setup() {
  pinMode(HOURS_BUTTON, INPUT_PULLUP);
  pinMode(MINUTES_BUTTON, INPUT_PULLUP);
  pinMode(SECONDS_BUTTON, INPUT_PULLUP);
  pinMode(START_BUTTON, INPUT_PULLUP);
  pinMode(RESET_BUTTON, INPUT_PULLUP);
  pinMode(RED_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);
  pinMode(BLUE_LED, OUTPUT);
  pinMode(RELAY_PIN, OUTPUT);

  // Initialize display
  display.setBrightness(0x0f); // Set display brightness
  display.showNumberDec(0, false); // Display 0 initially
}

void loop() {
  unsigned long currentMillis = millis();

  // Handle timer when it's active
  if (timerActive) {
    if (currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
      countdown();
    }
  }

  // Button handling
  handleButtons();

  // Update display
  updateDisplay();
}

void countdown() {
  if (fractions > 0) {
    fractions--;
  } else if (seconds > 0) {
    seconds--;
    fractions = 9;
  } else if (minutes > 0) {
    minutes--;
    seconds = 59;
    fractions = 9;
  } else if (hours > 0) {
    hours--;
    minutes = 59;
    seconds = 59;
    fractions = 9;
  } else {
    // Timer has reached zero
    timerActive = false;
    // Add code to trigger enlarger light here
  }
}

void handleButtons() {
  if (digitalRead(HOURS_BUTTON) == LOW) {
    hours = (hours + 1) % 24;
  }

  if (digitalRead(MINUTES_BUTTON) == LOW) {
    minutes = (minutes + 1) % 60;
  }

  if (digitalRead(SECONDS_BUTTON) == LOW) {
    seconds = (seconds + 1) % 60;
  }

  if (digitalRead(START_BUTTON) == LOW) {
    timerActive = true;
  }

  if (digitalRead(RESET_BUTTON) == LOW) {
    timerActive = false;
    hours = 0;
    minutes = 0;
    seconds = 0;
    fractions = 0;
    // Add code to turn off enlarger light and reset display here
  }
}

void updateDisplay() {
  int displayValue = (hours * 10000) + (minutes * 100) + seconds;
  display.showNumberDec(displayValue, false);
}

void setColor(byte red, byte green, byte blue) {
  analogWrite(RED_LED, red);
  analogWrite(GREEN_LED, green);
  analogWrite(BLUE_LED, blue);
}
